/*******************************************************************************
 * Copyright IBM Corp. and others 2018
 *
 * This program and the accompanying materials are made available under
 * the terms of the Eclipse Public License 2.0 which accompanies this
 * distribution and is available at https://www.eclipse.org/legal/epl-2.0/
 * or the Apache License, Version 2.0 which accompanies this distribution
 * and is available at https://www.apache.org/licenses/LICENSE-2.0.
 *
 * This Source Code may also be made available under the following Secondary
 * Licenses when the conditions for such availability set forth in the
 * Eclipse Public License, v. 2.0 are satisfied: GNU General Public License,
 * version 2 with the GNU Classpath Exception [1] and GNU General Public
 * License, version 2 with the OpenJDK Assembly Exception [2].
 *
 * [1] https://www.gnu.org/software/classpath/license.html
 * [2] https://openjdk.org/legal/assembly-exception.html
 *
 * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0-only WITH Classpath-exception-2.0 OR GPL-2.0-only WITH OpenJDK-assembly-exception-1.0
 *******************************************************************************/

#ifndef OMR_JITBUILDERRECORDER_TEXTFILE_INCL
#define OMR_JITBUILDERRECORDER_TEXTFILE_INCL

#include "ilgen/JitBuilderRecorder.hpp"

namespace TR {
class IlBuilder;
class MethodBuilder;
class IlType;
class IlValue;
} // namespace TR

namespace OMR {

class JitBuilderRecorderTextFile : public TR::JitBuilderRecorder {
public:
    JitBuilderRecorderTextFile(const TR::MethodBuilder *mb, const char *fileName);

    virtual ~JitBuilderRecorderTextFile() {}

    virtual void Close();
    virtual void String(const char * const string);
    virtual void Number(int8_t num);
    virtual void Number(int16_t num);
    virtual void Number(int32_t num);
    virtual void Number(int64_t num);
    virtual void Number(float num);
    virtual void Number(double num);
    virtual void ID(TypeID id);
    virtual void Statement(const char *s);
    virtual void Type(const TR::IlType *type);
    virtual void Value(const TR::IlValue *v);
    virtual void Builder(const TR::IlBuilder *b);
    virtual void Location(const void *location);
    virtual void EndStatement();
};

} // namespace OMR

#endif // !defined(OMR_JITBUILDERRECORDER_TEXTFILE_INCL)
